@@ -12,10 +12,9 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import http.client
 from oslo_log import log
 from oslo_utils import encodeutils
-import six
-from six.moves import http_client
 
 import keystone.conf
 from keystone.i18n import _
@@ -24,6 +23,8 @@ from keystone.i18n import _
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
+KEYSTONE_API_EXCEPTIONS = set([])
+
 # Tests use this to make exception message format errors fatal
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 
@@ -44,7 +45,21 @@ def _format_with_unicode_kwargs(msg_format, kwargs):
         return msg_format % kwargs
 
 
-class Error(Exception):
+class _KeystoneExceptionMeta(type):
+    """Automatically Register the Exceptions in 'KEYSTONE_API_EXCEPTIONS' list.
+
+    The `KEYSTONE_API_EXCEPTIONS` list is utilized by flask to register a
+    handler to emit sane details when the exception occurs.
+    """
+
+    def __new__(mcs, name, bases, class_dict):
+        """Create a new instance and register with KEYSTONE_API_EXCEPTIONS."""
+        cls = type.__new__(mcs, name, bases, class_dict)
+        KEYSTONE_API_EXCEPTIONS.add(cls)
+        return cls
+
+
+class Error(Exception, metaclass=_KeystoneExceptionMeta):
     """Base error class.
 
     Child classes should define an HTTP status code, title, and a
@@ -85,8 +100,8 @@ class ValidationError(Error):
                        " The server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
-    code = int(http_client.BAD_REQUEST)
-    title = http_client.responses[http_client.BAD_REQUEST]
+    code = int(http.client.BAD_REQUEST)
+    title = http.client.responses[http.client.BAD_REQUEST]
 
 
 class URLValidationError(ValidationError):
@@ -105,7 +120,7 @@ class PasswordRequirementsValidationError(PasswordValidationError):
 
 class PasswordHistoryValidationError(PasswordValidationError):
     message_format = _("The new password cannot be identical to a "
-                       "previous password. The total number which"
+                       "previous password. The total number which "
                        "includes the new password must be unique is "
                        "%(unique_count)s.")
 
@@ -137,8 +152,8 @@ class ValidationTimeStampError(Error):
                        " The server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
-    code = int(http_client.BAD_REQUEST)
-    title = http_client.responses[http_client.BAD_REQUEST]
+    code = int(http.client.BAD_REQUEST)
+    title = http.client.responses[http.client.BAD_REQUEST]
 
 
 class InvalidOperatorError(ValidationError):
@@ -152,8 +167,8 @@ class ValidationExpirationError(Error):
                        " The server could not comply with the request"
                        " since it is either malformed or otherwise"
                        " incorrect. The client is assumed to be in error.")
-    code = int(http_client.BAD_REQUEST)
-    title = http_client.responses[http_client.BAD_REQUEST]
+    code = int(http.client.BAD_REQUEST)
+    title = http.client.responses[http.client.BAD_REQUEST]
 
 
 class StringLengthExceeded(ValidationError):
@@ -175,8 +190,8 @@ class ApplicationCredentialValidationError(ValidationError):
 class CircularRegionHierarchyError(Error):
     message_format = _("The specified parent region %(parent_region_id)s "
                        "would create a circular region hierarchy.")
-    code = int(http_client.BAD_REQUEST)
-    title = http_client.responses[http_client.BAD_REQUEST]
+    code = int(http.client.BAD_REQUEST)
+    title = http.client.responses[http.client.BAD_REQUEST]
 
 
 class ForbiddenNotSecurity(Error):
@@ -187,8 +202,8 @@ class ForbiddenNotSecurity(Error):
 
     """
 
-    code = int(http_client.FORBIDDEN)
-    title = http_client.responses[http_client.FORBIDDEN]
+    code = int(http.client.FORBIDDEN)
+    title = http.client.responses[http.client.FORBIDDEN]
 
 
 class PasswordVerificationError(ForbiddenNotSecurity):
@@ -207,6 +222,11 @@ class ApplicationCredentialLimitExceeded(ForbiddenNotSecurity):
                        "maximum of %(limit)d already exceeded for user.")
 
 
+class CredentialLimitExceeded(ForbiddenNotSecurity):
+    message_format = _("Unable to create additional credentials, maximum "
+                       "of %(limit)d already exceeded for user.")
+
+
 class SecurityError(Error):
     """Security error exception.
 
@@ -248,7 +268,7 @@ class SecurityError(Error):
     def _build_message(self, message, **kwargs):
         """Only returns detailed messages in insecure_debug mode."""
         if message and CONF.insecure_debug:
-            if isinstance(message, six.string_types):
+            if isinstance(message, str):
                 # Only do replacement if message is string. The message is
                 # sometimes a different exception or bytes, which would raise
                 # TypeError.
@@ -262,18 +282,30 @@ class SecurityError(Error):
 
 class Unauthorized(SecurityError):
     message_format = _("The request you have made requires authentication.")
-    code = int(http_client.UNAUTHORIZED)
-    title = http_client.responses[http_client.UNAUTHORIZED]
+    code = int(http.client.UNAUTHORIZED)
+    title = http.client.responses[http.client.UNAUTHORIZED]
 
 
 class InsufficientAuthMethods(Error):
-    # NOTE(notmorgan): This is not a security error, this is meant to
-    # communicate real information back to the user.
+    # NOTE(adriant): This is an internal only error that is built into
+    # an auth receipt response.
     message_format = _("Insufficient auth methods received for %(user_id)s. "
                        "Auth Methods Provided: %(methods)s.")
     code = 401
     title = 'Unauthorized'
 
+    def __init__(self, message=None, user_id=None, methods=None):
+        methods_str = '[%s]' % ','.join(methods)
+        super(InsufficientAuthMethods, self).__init__(
+            message, user_id=user_id, methods=methods_str)
+
+        self.user_id = user_id
+        self.methods = methods
+
+
+class ReceiptNotFound(Unauthorized):
+    message_format = _("Could not find auth receipt: %(receipt_id)s.")
+
 
 class PasswordExpired(Unauthorized):
     message_format = _("The password is expired and needs to be changed for "
@@ -320,8 +352,8 @@ class AdditionalAuthRequired(AuthPluginException):
 class Forbidden(SecurityError):
     message_format = _("You are not authorized to perform the"
                        " requested action.")
-    code = int(http_client.FORBIDDEN)
-    title = http_client.responses[http_client.FORBIDDEN]
+    code = int(http.client.FORBIDDEN)
+    title = http.client.responses[http.client.FORBIDDEN]
 
 
 class ForbiddenAction(Forbidden):
@@ -362,8 +394,8 @@ class LimitTreeExceedError(Exception):
 
 class NotFound(Error):
     message_format = _("Could not find: %(target)s.")
-    code = int(http_client.NOT_FOUND)
-    title = http_client.responses[http_client.NOT_FOUND]
+    code = int(http.client.NOT_FOUND)
+    title = http.client.responses[http.client.NOT_FOUND]
 
 
 class EndpointNotFound(NotFound):
@@ -400,6 +432,10 @@ class DomainSpecificRoleNotWithinIdPDomain(Forbidden):
                        "the identity provider: %(identity_provider)s.")
 
 
+class DomainIdInvalid(ValidationError):
+    message_format = _("Domain ID does not conform to required UUID format.")
+
+
 class RoleAssignmentNotFound(NotFound):
     message_format = _("Could not find role assignment with role: "
                        "%(role_id)s, user or group: %(actor_id)s, "
@@ -516,11 +552,15 @@ class ApplicationCredentialNotFound(NotFound):
                        "%(application_credential_id)s.")
 
 
+class AccessRuleNotFound(NotFound):
+    message_format = _("Could not find Access Rule: %(access_rule_id)s.")
+
+
 class Conflict(Error):
     message_format = _("Conflict occurred attempting to store %(type)s -"
                        " %(details)s.")
-    code = int(http_client.CONFLICT)
-    title = http_client.responses[http_client.CONFLICT]
+    code = int(http.client.CONFLICT)
+    title = http.client.responses[http.client.CONFLICT]
 
 
 class UnexpectedError(SecurityError):
@@ -542,8 +582,8 @@ class UnexpectedError(SecurityError):
         return super(UnexpectedError, self)._build_message(
             message or self.debug_message_format, **kwargs)
 
-    code = int(http_client.INTERNAL_SERVER_ERROR)
-    title = http_client.responses[http_client.INTERNAL_SERVER_ERROR]
+    code = int(http.client.INTERNAL_SERVER_ERROR)
+    title = http.client.responses[http.client.INTERNAL_SERVER_ERROR]
 
 
 class TrustConsumeMaximumAttempt(UnexpectedError):
@@ -551,12 +591,6 @@ class TrustConsumeMaximumAttempt(UnexpectedError):
                              "acquire lock.")
 
 
-class CertificateFilesUnavailable(UnexpectedError):
-    debug_message_format = _("Expected signing certificates are not available "
-                             "on the server. Please check Keystone "
-                             "configuration.")
-
-
 class MalformedEndpoint(UnexpectedError):
     debug_message_format = _("Malformed endpoint URL (%(endpoint)s),"
                              " see ERROR log for details.")
@@ -587,15 +621,15 @@ class AssignmentTypeCalculationError(UnexpectedError):
 class NotImplemented(Error):
     message_format = _("The action you have requested has not"
                        " been implemented.")
-    code = int(http_client.NOT_IMPLEMENTED)
-    title = http_client.responses[http_client.NOT_IMPLEMENTED]
+    code = int(http.client.NOT_IMPLEMENTED)
+    title = http.client.responses[http.client.NOT_IMPLEMENTED]
 
 
 class Gone(Error):
     message_format = _("The service you have requested is no"
                        " longer available on this server.")
-    code = int(http_client.GONE)
-    title = http_client.responses[http_client.GONE]
+    code = int(http.client.GONE)
+    title = http.client.responses[http.client.GONE]
 
 
 class ConfigFileNotFound(UnexpectedError):
@@ -676,3 +710,15 @@ class CacheDeserializationError(Exception):
                 'obj': obj, 'data': data
             }
         )
+
+
+class ResourceUpdateForbidden(ForbiddenNotSecurity):
+    message_format = _('Unable to update immutable %(type)s resource: '
+                       '`%(resource_id)s. Set resource option "immutable" '
+                       'to false first.')
+
+
+class ResourceDeleteForbidden(ForbiddenNotSecurity):
+    message_format = _('Unable to delete immutable %(type)s resource: '
+                       '`%(resource_id)s. Set resource option "immutable" '
+                       'to false first.')
