@@ -18,6 +18,7 @@ import functools
 import os.path
 import re
 import sys
+import uuid
 import weakref
 
 import ldap.controls
@@ -25,9 +26,6 @@ import ldap.filter
 import ldappool
 from oslo_log import log
 from oslo_utils import reflection
-import six
-from six.moves import zip
-from six import PY2
 
 from keystone.common import driver_hints
 from keystone import exception
@@ -70,9 +68,9 @@ def utf8_encode(value):
     :returns: UTF-8 encoded version of value
     :raises TypeError: If value is not basestring
     """
-    if isinstance(value, six.text_type):
+    if isinstance(value, str):
         return _utf8_encoder(value)[0]
-    elif isinstance(value, six.binary_type):
+    elif isinstance(value, bytes):
         return value
     else:
         value_cls_name = reflection.get_class_name(
@@ -80,6 +78,7 @@ def utf8_encode(value):
         raise TypeError("value must be basestring, "
                         "not %s" % value_cls_name)
 
+
 _utf8_decoder = codecs.getdecoder('utf-8')
 
 
@@ -94,9 +93,18 @@ def utf8_decode(value):
     :returns: value as unicode
     :raises UnicodeDecodeError: for invalid UTF-8 encoding
     """
-    if isinstance(value, six.binary_type):
-        return _utf8_decoder(value)[0]
-    return six.text_type(value)
+    if isinstance(value, bytes):
+        try:
+            return _utf8_decoder(value)[0]
+        except UnicodeDecodeError:
+            # NOTE(lbragstad): We could be dealing with a UUID in byte form,
+            # which some LDAP implementations use.
+            uuid_byte_string_length = 16
+            if len(value) == uuid_byte_string_length:
+                return str(uuid.UUID(bytes_le=value))
+            else:
+                raise
+    return str(value)
 
 
 def py2ldap(val):
@@ -113,7 +121,7 @@ def py2ldap(val):
     if isinstance(val, bool):
         return u'TRUE' if val else u'FALSE'
     else:
-        return six.text_type(val)
+        return str(val)
 
 
 def enabled2py(val):
@@ -179,8 +187,8 @@ def convert_ldap_result(ldap_result):
                 LOG.debug('Unable to decode value for attribute %s', kind)
         py_result.append((dn, ldap_attrs))
     if at_least_one_referral:
-        LOG.debug(('Referrals were returned and ignored. Enable referral '
-                   'chasing in keystone.conf via [ldap] chase_referrals'))
+        LOG.debug('Referrals were returned and ignored. Enable referral '
+                  'chasing in keystone.conf via [ldap] chase_referrals')
 
     return py_result
 
@@ -332,8 +340,7 @@ def dn_startswith(descendant_dn, dn):
     return is_dn_equal(descendant_dn[-len(dn):], dn)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class LDAPHandler(object):
+class LDAPHandler(object, metaclass=abc.ABCMeta):
     """Abstract class which defines methods for a LDAP API provider.
 
     Native Keystone values cannot be passed directly into and from the
@@ -516,14 +523,7 @@ class PythonLDAPHandler(LDAPHandler):
                                     debug_level=debug_level,
                                     timeout=conn_timeout)
 
-        if PY2:
-            # NOTE: Once https://github.com/python-ldap/python-ldap/issues/249
-            # is released, we can pass bytes_strictness='warn' as a parameter
-            # to ldap.initialize instead of setting it after ldap.initialize.
-            self.conn = ldap.initialize(url, bytes_mode=False)
-            self.conn.bytes_strictness = 'warn'
-        else:
-            self.conn = ldap.initialize(url)
+        self.conn = ldap.initialize(url)
         self.conn.protocol_version = ldap.VERSION3
 
         if alias_dereferencing is not None:
@@ -1272,7 +1272,7 @@ class BaseLdap(object):
     def _id_to_dn_string(self, object_id):
         return u'%s=%s,%s' % (self.id_attr,
                               ldap.dn.escape_dn_chars(
-                                  six.text_type(object_id)),
+                                  str(object_id)),
                               self.tree_dn)
 
     def _id_to_dn(self, object_id):
@@ -1284,7 +1284,7 @@ class BaseLdap(object):
                 u'(&(%(id_attr)s=%(id)s)(objectclass=%(objclass)s))' %
                 {'id_attr': self.id_attr,
                  'id': ldap.filter.escape_filter_chars(
-                     six.text_type(object_id)),
+                     str(object_id)),
                  'objclass': self.object_class},
                 attrlist=DN_ONLY)
         if search_result:
@@ -1316,7 +1316,7 @@ class BaseLdap(object):
                 if len(id_list) > 1:
                     message = ('In order to keep backward compatibility, in '
                                'the case of multivalued ids, we are '
-                               'returning the first id %(id_attr) in the '
+                               'returning the first id %(id_attr)s in the '
                                'DN.') % ({'id_attr': id_list[0]})
                     LOG.warning(message)
                 return id_list[0]
@@ -1478,7 +1478,7 @@ class BaseLdap(object):
                  u'(objectClass=%(object_class)s))'
                  % {'id_attr': self.id_attr,
                     'id': ldap.filter.escape_filter_chars(
-                        six.text_type(object_id)),
+                        str(object_id)),
                     'filter': (ldap_filter or self.ldap_filter or ''),
                     'object_class': self.object_class})
         with self.get_connection() as conn:
@@ -1579,7 +1579,7 @@ class BaseLdap(object):
     def get_by_name(self, name, ldap_filter=None):
         query = (u'(%s=%s)' % (self.attribute_mapping['name'],
                                ldap.filter.escape_filter_chars(
-                                   six.text_type(name))))
+                                   str(name))))
         res = self.get_all(query)
         try:
             return res[0]
@@ -1781,6 +1781,7 @@ class EnabledEmuMixIn(BaseLdap):
 
     DEFAULT_GROUP_OBJECTCLASS = 'groupOfNames'
     DEFAULT_MEMBER_ATTRIBUTE = 'member'
+    DEFAULT_GROUP_MEMBERS_ARE_IDS = False
 
     def __init__(self, conf):
         super(EnabledEmuMixIn, self).__init__(conf)
@@ -1797,9 +1798,11 @@ class EnabledEmuMixIn(BaseLdap):
         if not self.use_group_config:
             self.member_attribute = self.DEFAULT_MEMBER_ATTRIBUTE
             self.group_objectclass = self.DEFAULT_GROUP_OBJECTCLASS
+            self.group_members_are_ids = self.DEFAULT_GROUP_MEMBERS_ARE_IDS
         else:
             self.member_attribute = conf.ldap.group_member_attribute
             self.group_objectclass = conf.ldap.group_objectclass
+            self.group_members_are_ids = conf.ldap.group_members_are_ids
 
         if not self.enabled_emulation_dn:
             naming_attr_name = 'cn'
@@ -1815,10 +1818,19 @@ class EnabledEmuMixIn(BaseLdap):
                            naming_rdn[1])
         self.enabled_emulation_naming_attr = naming_attr
 
-    def _get_enabled(self, object_id, conn):
-        dn = self._id_to_dn(object_id)
+    def _id_to_member_attribute_value(self, object_id):
+        """Convert id to value expected by member_attribute."""
+        if self.group_members_are_ids:
+            return object_id
+        return self._id_to_dn(object_id)
+
+    def _is_id_enabled(self, object_id, conn):
+        member_attr_val = self._id_to_member_attribute_value(object_id)
+        return self._is_member_enabled(member_attr_val, conn)
+
+    def _is_member_enabled(self, member_attr_val, conn):
         query = '(%s=%s)' % (self.member_attribute,
-                             ldap.filter.escape_filter_chars(dn))
+                             ldap.filter.escape_filter_chars(member_attr_val))
         try:
             enabled_value = conn.search_s(self.enabled_emulation_dn,
                                           ldap.SCOPE_BASE,
@@ -1829,24 +1841,26 @@ class EnabledEmuMixIn(BaseLdap):
             return bool(enabled_value)
 
     def _add_enabled(self, object_id):
+        member_attr_val = self._id_to_member_attribute_value(object_id)
         with self.get_connection() as conn:
-            if not self._get_enabled(object_id, conn):
+            if not self._is_member_enabled(member_attr_val, conn):
                 modlist = [(ldap.MOD_ADD,
                             self.member_attribute,
-                            [self._id_to_dn(object_id)])]
+                            [member_attr_val])]
                 try:
                     conn.modify_s(self.enabled_emulation_dn, modlist)
                 except ldap.NO_SUCH_OBJECT:
                     attr_list = [('objectClass', [self.group_objectclass]),
                                  (self.member_attribute,
-                                  [self._id_to_dn(object_id)]),
+                                  [member_attr_val]),
                                  self.enabled_emulation_naming_attr]
                     conn.add_s(self.enabled_emulation_dn, attr_list)
 
     def _remove_enabled(self, object_id):
+        member_attr_val = self._id_to_member_attribute_value(object_id)
         modlist = [(ldap.MOD_DELETE,
                     self.member_attribute,
-                    [self._id_to_dn(object_id)])]
+                    [member_attr_val])]
         with self.get_connection() as conn:
             try:
                 conn.modify_s(self.enabled_emulation_dn, modlist)
@@ -1871,7 +1885,7 @@ class EnabledEmuMixIn(BaseLdap):
             ref = super(EnabledEmuMixIn, self).get(object_id, ldap_filter)
             if ('enabled' not in self.attribute_ignore and
                     self.enabled_emulation):
-                ref['enabled'] = self._get_enabled(object_id, conn)
+                ref['enabled'] = self._is_id_enabled(object_id, conn)
             return ref
 
     def get_all(self, ldap_filter=None, hints=None):
@@ -1883,7 +1897,7 @@ class EnabledEmuMixIn(BaseLdap):
                         if x[0] != self.enabled_emulation_dn]
             with self.get_connection() as conn:
                 for obj_ref in obj_list:
-                    obj_ref['enabled'] = self._get_enabled(
+                    obj_ref['enabled'] = self._is_id_enabled(
                         obj_ref['id'], conn)
             return obj_list
         else:
