@@ -14,8 +14,6 @@
 
 """Main entry point into the OAuth1 service."""
 
-from __future__ import absolute_import
-
 import uuid
 
 import oauthlib.common
@@ -89,17 +87,22 @@ def validate_oauth_params(query_string):
     params_fitered = {k: v for k, v in params if not k.startswith('oauth_')}
     if params_fitered:
         if 'error' in params_fitered:
-            msg = _(
+            msg = (
                 'Validation failed with errors: %(error)s, detail '
                 'message is: %(desc)s.') % {
                     'error': params_fitered['error'],
                     'desc': params_fitered['error_description']}
+            tr_msg = _('Validation failed with errors: %(error)s, detail '
+                       'message is: %(desc)s.') % {
+                'error': params_fitered['error'],
+                'desc': params_fitered['error_description']}
         else:
-            msg = _(
-                'Unknown parameters found, '
-                'please provide only oauth parameters.')
+            msg = ('Unknown parameters found,'
+                   'please provide only oauth parameters.')
+            tr_msg = _('Unknown parameters found,'
+                       'please provide only oauth parameters.')
         LOG.warning(msg)
-        raise exception.ValidationError(message=msg)
+        raise exception.ValidationError(message=tr_msg)
 
 
 class Manager(manager.Manager):
