@@ -22,13 +22,10 @@ import flask
 from flask import blueprints
 import flask_restful
 import flask_restful.utils
+import http.client
 from oslo_log import log
+from oslo_log import versionutils
 from oslo_serialization import jsonutils
-from pycadf import cadftaxonomy as taxonomy
-from pycadf import host
-from pycadf import resource
-import six
-from six.moves import http_client
 
 from keystone.common import authorization
 from keystone.common import context
@@ -38,6 +35,8 @@ from keystone.common.rbac_enforcer import enforcer
 from keystone.common import utils
 import keystone.conf
 from keystone import exception
+from keystone.i18n import _
+from keystone import notifications
 
 
 # NOTE(morgan): Capture the relevant part of the flask url route rule for
@@ -90,15 +89,21 @@ def construct_resource_map(resource, url, resource_kwargs, alternate_urls=None,
                             Additional keyword arguments not specified above
                             will be passed as-is to
                             :meth:`flask.Flask.add_url_rule`.
-    :param alternate_urls: An iterable (list) of urls that also map to the
-                           resource. These are used to ensure API compat when
-                           a "new" path is more correct for the API but old
-                           paths must continue to work. Example:
+    :param alternate_urls: An iterable (list) of dictionaries containing urls
+                           and associated json home REL data. Each element is
+                           expected to be a dictionary with a 'url' key and an
+                           optional 'json_home' key for a 'JsonHomeData' named
+                           tuple  These urls will also map to the resource.
+                           These are used to ensure API compatibility when a
+                           "new" path is more correct for the API but old paths
+                           must continue to work. Example:
                            `/auth/domains` being the new path for
                            `/OS-FEDERATION/domains`. The `OS-FEDERATION` part
-                           would be listed as an alternate url. These are not
-                           added to the JSON Home Document.
-    :type: any iterable or None
+                           would be listed as an alternate url. If a
+                           'json_home' key is provided, the original path
+                           with the new json_home data will be added to the
+                           JSON Home Document.
+    :type: iterable or None
     :param rel:
     :type rel: str or None
     :param status: JSON Home API Status, e.g. "STABLE"
@@ -137,42 +142,25 @@ def _assert_rbac_enforcement_called(resp):
     # assert is intended to be used to ensure code during development works
     # as expected, it is fine to be optimized out with `python -O`
     msg = ('PROGRAMMING ERROR: enforcement (`keystone.common.rbac_enforcer.'
-           'enforcer.RBACKEnforcer.enforce_call()`) has not been called; API '
+           'enforcer.RBACEnforcer.enforce_call()`) has not been called; API '
            'is unenforced.')
     g = flask.g
-    assert getattr(g, enforcer._ENFORCEMENT_CHECK_ATTR, False), msg  # nosec
+    # NOTE(morgan): OPTIONS is a special case and is handled by flask
+    # internally. We should never be enforcing on OPTIONS calls.
+    if flask.request.method != 'OPTIONS':
+        assert getattr(  # nosec
+            g, enforcer._ENFORCEMENT_CHECK_ATTR, False), msg  # nosec
     return resp
 
 
 def _remove_content_type_on_204(resp):
     # Remove content-type if the resp is 204.
-    if resp.status_code == http_client.NO_CONTENT:
+    if resp.status_code == http.client.NO_CONTENT:
         resp.headers.pop('content-type', None)
     return resp
 
 
-def build_audit_initiator():
-    """A pyCADF initiator describing the current authenticated context."""
-    pycadf_host = host.Host(address=flask.request.remote_addr,
-                            agent=str(flask.request.user_agent))
-    initiator = resource.Resource(typeURI=taxonomy.ACCOUNT_USER,
-                                  host=pycadf_host)
-    oslo_context = flask.request.environ.get(context.REQUEST_CONTEXT_ENV)
-    if oslo_context.user_id:
-        initiator.id = utils.resource_uuid(oslo_context.user_id)
-        initiator.user_id = oslo_context.user_id
-
-    if oslo_context.project_id:
-        initiator.project_id = oslo_context.project_id
-
-    if oslo_context.domain_id:
-        initiator.domain_id = oslo_context.domain_id
-
-    return initiator
-
-
-@six.add_metaclass(abc.ABCMeta)
-class APIBase(object):
+class APIBase(object, metaclass=abc.ABCMeta):
 
     @property
     @abc.abstractmethod
@@ -317,6 +305,7 @@ class APIBase(object):
             c_key = getattr(r, 'collection_key', None)
             m_key = getattr(r, 'member_key', None)
             r_pfx = getattr(r, 'api_prefix', None)
+
             if not c_key or not m_key:
                 LOG.debug('Unable to add resource %(resource)s to API '
                           '%(name)s, both `member_key` and `collection_key` '
@@ -336,13 +325,20 @@ class APIBase(object):
             # NOTE(morgan): The Prefix is automatically added by the API, so
             # we do not add it to the paths here.
             collection_path = '/%s' % c_key
-            entity_path = '/%(collection)s/<string:%(member)s_id>' % {
-                'collection': c_key, 'member': m_key}
+            if getattr(r, '_id_path_param_name_override', None):
+                # The member_key doesn't match the "id" key in the url, make
+                # sure to use the correct path-key for ID.
+                member_id_key = getattr(r, '_id_path_param_name_override')
+            else:
+                member_id_key = '%(member_key)s_id' % {'member_key': m_key}
+
+            entity_path = '/%(collection)s/<string:%(member)s>' % {
+                'collection': c_key, 'member': member_id_key}
             # NOTE(morgan): The json-home form of the entity path is different
             # from the flask-url routing form. Must also include the prefix
-            jh_e_path = '%(pfx)s/%(e_path)s' % {
+            jh_e_path = _URL_SUBST.sub('{\\1}', '%(pfx)s/%(e_path)s' % {
                 'pfx': self._api_url_prefix,
-                'e_path': _URL_SUBST.sub('{\\1}', entity_path).lstrip('/')}
+                'e_path': entity_path.lstrip('/')})
 
             LOG.debug(
                 'Adding standard routes to API %(name)s for `%(resource)s` '
@@ -360,16 +356,37 @@ class APIBase(object):
                 json_home.build_v3_resource_relation)
             resource_rel_status = getattr(
                 r, 'json_home_resource_status', None)
-            collection_rel = resource_rel_func(resource_name=c_key)
+            collection_rel_resource_name = getattr(
+                r, 'json_home_collection_resource_name_override', c_key)
+            collection_rel = resource_rel_func(
+                resource_name=collection_rel_resource_name)
             # NOTE(morgan): Add the prefix explicitly for JSON Home documents
             # to the collection path.
-            rel_data = {'href': '%(pfx)s%(collection_path)s' % {
+            href_val = '%(pfx)s%(collection_path)s' % {
                 'pfx': self._api_url_prefix,
                 'collection_path': collection_path}
-            }
 
-            entity_rel = resource_rel_func(resource_name=m_key)
-            id_str = '%s_id' % m_key
+            # If additional parameters exist in the URL, add them to the
+            # href-vars dict.
+            additional_params = getattr(
+                r, 'json_home_additional_parameters', {})
+
+            if additional_params:
+                # NOTE(morgan): Special case, we have 'additional params' which
+                # means we know the params are in the "prefix". This guarantees
+                # the correct data in the json_home document with href-template
+                # and href-vars even on the "collection" entry
+                rel_data = dict()
+                rel_data['href-template'] = _URL_SUBST.sub('{\\1}', href_val)
+                rel_data['href-vars'] = additional_params
+            else:
+                rel_data = {'href': href_val}
+            member_rel_resource_name = getattr(
+                r, 'json_home_member_resource_name_override', m_key)
+
+            entity_rel = resource_rel_func(
+                resource_name=member_rel_resource_name)
+            id_str = member_id_key
 
             parameter_rel_func = getattr(
                 r, 'json_home_parameter_rel_func',
@@ -378,6 +395,10 @@ class APIBase(object):
             entity_rel_data = {'href-template': jh_e_path,
                                'href-vars': {id_str: id_param_rel}}
 
+            if additional_params:
+                entity_rel_data.setdefault('href-vars', {}).update(
+                    additional_params)
+
             if resource_rel_status is not None:
                 json_home.Status.update_resource_data(
                     rel_data, resource_rel_status)
@@ -392,19 +413,33 @@ class APIBase(object):
     def _add_mapped_resources(self):
         # Add resource mappings, non-standard resource connections
         for r in self.resource_mapping:
+            alt_url_json_home_data = []
             LOG.debug(
                 'Adding resource routes to API %(name)s: '
                 '[%(url)r %(kwargs)r]',
                 {'name': self._name, 'url': r.url, 'kwargs': r.kwargs})
-            self.api.add_resource(r.resource, r.url, **r.kwargs)
+            urls = [r.url]
             if r.alternate_urls is not None:
-                LOG.debug(
-                    'Adding additional resource routes (alternate) to API'
-                    '%(name)s: [%(urls)r %(kwargs)r]',
-                    {'name': self._name, 'urls': r.alternate_urls,
-                     'kwargs': r.kwargs})
-                self.api.add_resource(r.resource, *r.alternate_urls,
-                                      **r.kwargs)
+                for element in r.alternate_urls:
+                    if self._api_url_prefix:
+                        LOG.debug(
+                            'Unable to add additional resource route '
+                            '`%(route)s` to API %(name)s because API has a '
+                            'URL prefix. Only APIs without explicit prefixes '
+                            'can have alternate URL routes added.',
+                            {'route': element['url'], 'name': self._name}
+                        )
+                        continue
+                    LOG.debug(
+                        'Adding additional resource route (alternate) to API '
+                        '%(name)s: [%(url)r %(kwargs)r]',
+                        {'name': self._name, 'url': element['url'],
+                         'kwargs': r.kwargs})
+                    urls.append(element['url'])
+                    if element.get('json_home'):
+                        alt_url_json_home_data.append(element['json_home'])
+            # Add all URL routes at once.
+            self.api.add_resource(r.resource, *urls, **r.kwargs)
 
             # Build the JSON Home data and add it to the relevant JSON Home
             # Documents for explicit JSON Home data.
@@ -427,6 +462,12 @@ class APIBase(object):
                     r.json_home_data.rel,
                     resource_data)
 
+                for element in alt_url_json_home_data:
+                    # Append the "new" path (resource) data with the old rel
+                    # reference.
+                    json_home.JsonHomeResources.append_resource(
+                        element.rel, resource_data)
+
     def _register_before_request_functions(self, functions=None):
         """Register functions to be executed in the `before request` phase.
 
@@ -531,31 +572,42 @@ class APIBase(object):
         return inst
 
 
-class ResourceBase(flask_restful.Resource):
+class _AttributeRaisesError(object):
+    # NOTE(morgan): This is a special case class that exists to effectively
+    # create a @classproperty style function. We use __get__ to raise the
+    # exception.
 
-    collection_key = None
-    member_key = None
+    def __init__(self, name):
+        self.__msg = 'PROGRAMMING ERROR: `self.{name}` is not set.'.format(
+            name=name)
+
+    def __get__(self, instance, owner):
+        raise ValueError(self.__msg)
+
+
+class ResourceBase(flask_restful.Resource):
+    collection_key = _AttributeRaisesError(name='collection_key')
+    member_key = _AttributeRaisesError(name='member_key')
+    _public_parameters = frozenset([])
     # NOTE(morgan): This must match the string on the API the resource is
     # registered to.
     api_prefix = ''
+    _id_path_param_name_override = None
 
     method_decorators = []
 
-    def __init__(self):
-        super(ResourceBase, self).__init__()
-        if self.collection_key is None:
-            raise ValueError('PROGRAMMING ERROR: `self.collection_key` '
-                             'cannot be `None`.')
-        if self.member_key is None:
-            raise ValueError('PROGRAMMING ERROR: `self.member_key` cannot '
-                             'be `None`.')
-
     @staticmethod
     def _assign_unique_id(ref):
         ref = ref.copy()
         ref['id'] = uuid.uuid4().hex
         return ref
 
+    @staticmethod
+    def _validate_id_format(id):
+        uval = uuid.UUID(id).hex
+        if uval != id:
+            raise ValueError('badly formed hexadecimal UUID value')
+
     @classmethod
     def _require_matching_id(cls, ref):
         """Ensure the value matches the reference's ID, if any."""
@@ -566,7 +618,26 @@ class ResourceBase(flask_restful.Resource):
             raise exception.ValidationError('Cannot change ID')
 
     @classmethod
-    def wrap_collection(cls, refs, hints=None):
+    def filter_params(cls, ref):
+        """Remove unspecified parameters from the dictionary.
+
+        This function removes unspecified parameters from the dictionary.
+        This method checks only root-level keys from a ref dictionary.
+
+        :param ref: a dictionary representing deserialized response to be
+                    serialized
+        """
+        # NOTE(morgan): if _public_parameters is empty, do nothing. We do not
+        # filter if we do not have an explicit white-list to work from.
+        if cls._public_parameters:
+            ref_keys = set(ref.keys())
+            blocked_keys = ref_keys - cls._public_parameters
+            for blocked_param in blocked_keys:
+                del ref[blocked_param]
+        return ref
+
+    @classmethod
+    def wrap_collection(cls, refs, hints=None, collection_name=None):
         """Wrap a collection, checking for filtering and pagination.
 
         Returns the wrapped collection, which includes:
@@ -579,6 +650,10 @@ class ResourceBase(flask_restful.Resource):
         :param hints: list hints, containing any relevant filters and limit.
                       Any filters already satisfied by managers will have been
                       removed
+        :param collection_name: optional override for the 'collection key'
+                                class attribute. This is to be used when
+                                wrapping a collection for a different api,
+                                e.g. 'roles' from the 'trust' api.
         """
         # Check if there are any filters in hints that were not handled by
         # the drivers. The driver will not have paginated or limited the
@@ -589,13 +664,13 @@ class ResourceBase(flask_restful.Resource):
 
         list_limited, refs = cls.limit(refs, hints)
 
-        for ref in refs:
-            cls._add_self_referential_link(ref)
+        collection = collection_name or cls.collection_key
 
-        container = {cls.collection_key: refs}
-        pfx = getattr(cls, 'api_prefix', '').lstrip('/')
-        parts = [p for p in (full_url(), 'v3', pfx, cls.collection_key) if p]
-        self_url = '/'.join(parts)
+        for ref in refs:
+            cls._add_self_referential_link(ref, collection_name=collection)
+
+        container = {collection: refs}
+        self_url = full_url(flask.request.environ['PATH_INFO'])
         container['links'] = {
             'next': None,
             'self': self_url,
@@ -607,17 +682,26 @@ class ResourceBase(flask_restful.Resource):
         return container
 
     @classmethod
-    def wrap_member(cls, ref):
-        cls._add_self_referential_link(ref)
-        return {cls.member_key: ref}
+    def wrap_member(cls, ref, collection_name=None, member_name=None):
+        cls._add_self_referential_link(ref, collection_name)
+        return {member_name or cls.member_key: ref}
 
     @classmethod
-    def _add_self_referential_link(cls, ref):
-        collection_element = cls.collection_key
+    def _add_self_referential_link(cls, ref, collection_name=None):
+        collection_element = collection_name or cls.collection_key
         if cls.api_prefix:
             api_prefix = cls.api_prefix.lstrip('/').rstrip('/')
-            collection_element = '/'.join([api_prefix, cls.collection_key])
-        self_link = '/'.join([base_url(), 'v3', collection_element, ref['id']])
+            # ensure we have substituted the flask-arg specification
+            # to the "keystone" mechanism, then format the string
+            api_prefix = _URL_SUBST.sub('{\\1}', api_prefix)
+            if flask.request.view_args:
+                # if a prefix has substitutions it is *required* that the
+                # values are passed as view_args to the HTTP action method
+                # (e.g. head/get/post/...).
+                api_prefix = api_prefix.format(**flask.request.view_args)
+            collection_element = '/'.join(
+                [api_prefix, collection_name or cls.collection_key])
+        self_link = base_url(path='/'.join([collection_element, ref['id']]))
         ref.setdefault('links', {})['self'] = self_link
 
     @classmethod
@@ -695,7 +779,42 @@ class ResourceBase(flask_restful.Resource):
 
         As a property.
         """
-        return build_audit_initiator()
+        return notifications.build_audit_initiator()
+
+    @staticmethod
+    def query_filter_is_true(filter_name):
+        """Determine if bool query param is 'True'.
+
+        We treat this the same way as we do for policy
+        enforcement:
+
+        {bool_param}=0 is treated as False
+
+        Any other value is considered to be equivalent to
+        True, including the absence of a value (but existence
+        as a parameter).
+
+        False Examples for param named `p`:
+
+           * http://host/url
+           * http://host/url?p=0
+
+        All other forms of the param 'p' would be result in a True value
+        including: `http://host/url?param`.
+        """
+        val = False
+        if filter_name in flask.request.args:
+            filter_value = flask.request.args.get(filter_name)
+            if (isinstance(filter_value, str) and
+                    filter_value == '0'):
+                val = False
+            else:
+                val = True
+        return val
+
+    @property
+    def request_body_json(self):
+        return flask.request.get_json(silent=True, force=True) or {}
 
     @staticmethod
     def build_driver_hints(supported_filters):
@@ -710,7 +829,7 @@ class ResourceBase(flask_restful.Resource):
         if not flask.request.args:
             return hints
 
-        for key, value in flask.request.args.items():
+        for key, value in flask.request.args.items(multi=True):
             # Check if this is an exact filter
             if supported_filters is None or key in supported_filters:
                 hints.add_filter(key, value)
@@ -792,8 +911,95 @@ class ResourceBase(flask_restful.Resource):
     def _normalize_arg(arg):
         return arg.replace(':', '_').replace('-', '_')
 
+    @classmethod
+    def _get_domain_id_for_list_request(cls):
+        """Get the domain_id for a v3 list call.
 
-def base_url():
+        If we running with multiple domain drivers, then the caller must
+        specify a domain_id either as a filter or as part of the token scope.
+
+        """
+        if not CONF.identity.domain_specific_drivers_enabled:
+            # We don't need to specify a domain ID in this case
+            return
+
+        domain_id = flask.request.args.get('domain_id')
+        if domain_id:
+            return domain_id
+
+        token_ref = cls.get_token_ref()
+
+        if token_ref.domain_scoped:
+            return token_ref.domain_id
+        elif token_ref.project_scoped:
+            return token_ref.project_domain['id']
+        elif token_ref.system_scoped:
+            return
+        else:
+            msg = 'No domain information specified as part of list request'
+            tr_msg = _('No domain information specified as part of list '
+                       'request')
+            LOG.warning(msg)
+            raise exception.Unauthorized(tr_msg)
+
+    @classmethod
+    def get_token_ref(cls):
+        """Retrieve KeystoneToken object from the auth context and returns it.
+
+        :raises keystone.exception.Unauthorized: If auth context cannot be
+                                                 found.
+        :returns: The KeystoneToken object.
+        """
+        try:
+            # Retrieve the auth context that was prepared by
+            # AuthContextMiddleware.
+
+            auth_context = flask.request.environ.get(
+                authorization.AUTH_CONTEXT_ENV, {})
+            return auth_context['token']
+        except KeyError:
+            LOG.warning("Couldn't find the auth context.")
+            raise exception.Unauthorized()
+
+    @classmethod
+    def _normalize_domain_id(cls, ref):
+        """Fill in domain_id if not specified in a v3 call."""
+        if not ref.get('domain_id'):
+            oslo_ctx = flask.request.environ.get(
+                context.REQUEST_CONTEXT_ENV, None)
+            if oslo_ctx and oslo_ctx.domain_id:
+                # Domain Scoped Token Scenario.
+                ref['domain_id'] = oslo_ctx.domain_id
+            elif oslo_ctx.is_admin:
+                # Legacy "shared" admin token Scenario
+                raise exception.ValidationError(
+                    _('You have tried to create a resource using the admin '
+                      'token. As this token is not within a domain you must '
+                      'explicitly include a domain for this resource to '
+                      'belong to.'))
+            else:
+                # TODO(henry-nash): We should issue an exception here since if
+                # a v3 call does not explicitly specify the domain_id in the
+                # entity, it should be using a domain scoped token.  However,
+                # the current tempest heat tests issue a v3 call without this.
+                # This is raised as bug #1283539.  Once this is fixed, we
+                # should remove the line below and replace it with an error.
+                #
+                # Ahead of actually changing the code to raise an exception, we
+                # issue a deprecation warning.
+                versionutils.report_deprecated_feature(
+                    LOG,
+                    'Not specifying a domain during a create user, group or '
+                    'project call, and relying on falling back to the '
+                    'default domain, is deprecated as of Liberty. There is no '
+                    'plan to remove this compatibility, however, future API '
+                    'versions may remove this, so please specify the domain '
+                    'explicitly or use a domain-scoped token.')
+                ref['domain_id'] = CONF.identity.default_domain_id
+        return ref
+
+
+def base_url(path=''):
     url = CONF['public_endpoint']
 
     if url:
@@ -815,17 +1021,32 @@ def base_url():
         # production environment.
         url = 'http://localhost:%d' % CONF.eventlet_server.public_port
 
-    return url.rstrip('/')
+    if path:
+        # Cleanup leading /v3 if needed.
+        path = path.rstrip('/').lstrip('/')
+        if path.startswith('v3'):
+            path = path[2:].lstrip('/')
+
+    url = url.rstrip('/')
+    url = '/'.join([p for p in (url, 'v3', path) if p])
+    return url
 
 
-def full_url():
-    subs = {'url': base_url(), 'query_string': ''}
+def full_url(path=''):
+    subs = {'url': base_url(path), 'query_string': ''}
     qs = flask.request.environ.get('QUERY_STRING')
     if qs:
         subs['query_string'] = '?%s' % qs
     return '%(url)s%(query_string)s' % subs
 
 
+def set_unenforced_ok():
+    # Does the work for unenforced_api. This must be used outside of a
+    # decorator in some limited, such as when a ValidationError is raised up
+    # from a "before_request" function (body_json checker is a prime example)
+    setattr(flask.g, enforcer._ENFORCEMENT_CHECK_ATTR, True)
+
+
 def unenforced_api(f):
     """Decorate a resource method to mark is as an unenforced API.
 
@@ -839,6 +1060,6 @@ def unenforced_api(f):
     """
     @functools.wraps(f)
     def wrapper(*args, **kwargs):
-        setattr(flask.g, enforcer._ENFORCEMENT_CHECK_ATTR, True)
+        set_unenforced_ok()
         return f(*args, **kwargs)
     return wrapper
