@@ -20,7 +20,7 @@ from keystone.common.rbac_enforcer import policy
 class Policy(fixtures.Fixture):
     """A fixture for working with policy configuration."""
 
-    def __init__(self, policy_file, config_fixture):
+    def __init__(self, config_fixture, policy_file=None):
         self._policy_file = policy_file
         self._config_fixture = config_fixture
 
@@ -29,4 +29,5 @@ class Policy(fixtures.Fixture):
         opts.set_defaults(self._config_fixture.conf)
         self._config_fixture.config(group='oslo_policy',
                                     policy_file=self._policy_file)
+        policy._ENFORCER.suppress_deprecation_warnings = True
         self.addCleanup(policy.reset)
