@@ -12,12 +12,13 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import datetime
+from unittest import mock
 import uuid
 
-import mock
+import freezegun
 from oslo_db import exception as db_exception
 from oslo_db import options
-from six.moves import range
 import sqlalchemy
 from sqlalchemy import exc
 from testtools import matchers
@@ -25,6 +26,7 @@ from testtools import matchers
 from keystone.common import driver_hints
 from keystone.common import provider_api
 from keystone.common import sql
+from keystone.common.sql import core
 import keystone.conf
 from keystone.credential.providers import fernet as credential_provider
 from keystone import exception
@@ -66,14 +68,126 @@ class SqlTests(unit.SQLDriverOverrides, unit.TestCase):
         return config_files
 
 
+class DataTypeRoundTrips(SqlTests):
+    def test_json_blob_roundtrip(self):
+        """Test round-trip of a JSON data structure with JsonBlob."""
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [sqlalchemy.literal({"key": "value"}, type_=core.JsonBlob)]
+                )
+            )
+
+        self.assertEqual({"key": "value"}, val)
+
+    def test_json_blob_sql_null(self):
+        """Test that JsonBlob can accommodate a SQL NULL value in a result set.
+
+        SQL NULL may be handled by JsonBlob in the case where a table is
+        storing NULL in a JsonBlob column, as several models use this type
+        in a column that is nullable.   It also comes back when the column
+        is left NULL from being in an OUTER JOIN.  In Python, this means
+        the None constant is handled by the datatype.
+
+        """
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [sqlalchemy.cast(sqlalchemy.null(), type_=core.JsonBlob)]
+                )
+            )
+
+        self.assertIsNone(val)
+
+    def test_json_blob_python_none(self):
+        """Test that JsonBlob round-trips a Python None.
+
+        This is where JSON datatypes get a little nutty, in that JSON has
+        a 'null' keyword, and JsonBlob right now will persist Python None
+        as the json string 'null', not SQL NULL.
+
+        """
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [sqlalchemy.literal(None, type_=core.JsonBlob)]
+                )
+            )
+
+        self.assertIsNone(val)
+
+    def test_json_blob_python_none_renders(self):
+        """Test that JsonBlob actually renders JSON 'null' for Python None."""
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.cast(
+                            sqlalchemy.literal(None, type_=core.JsonBlob),
+                            sqlalchemy.String,
+                        )
+                    ]
+                )
+            )
+
+        self.assertEqual("null", val)
+
+    def test_datetimeint_roundtrip(self):
+        """Test round-trip of a Python datetime with DateTimeInt."""
+        with sql.session_for_read() as session:
+            datetime_value = datetime.datetime(2019, 5, 15, 10, 17, 55)
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.literal(
+                            datetime_value, type_=core.DateTimeInt
+                        ),
+                    ]
+                )
+            )
+
+        self.assertEqual(datetime_value, val)
+
+    def test_datetimeint_persistence(self):
+        """Test integer persistence with DateTimeInt."""
+        with sql.session_for_read() as session:
+            datetime_value = datetime.datetime(2019, 5, 15, 10, 17, 55)
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.cast(
+                            sqlalchemy.literal(
+                                datetime_value, type_=core.DateTimeInt
+                            ),
+                            sqlalchemy.Integer
+                        )
+                    ]
+                )
+            )
+
+        self.assertEqual(1557915475000000, val)
+
+    def test_datetimeint_python_none(self):
+        """Test round-trip of a Python None with DateTimeInt."""
+        with sql.session_for_read() as session:
+            val = session.scalar(
+                sqlalchemy.select(
+                    [
+                        sqlalchemy.literal(None, type_=core.DateTimeInt),
+                    ]
+                )
+            )
+
+        self.assertIsNone(val)
+
+
 class SqlModels(SqlTests):
 
-    def select_table(self, name):
+    def load_table(self, name):
         table = sqlalchemy.Table(name,
                                  sql.ModelBase.metadata,
                                  autoload=True)
-        s = sqlalchemy.select([table])
-        return s
+        return table
 
     def assertExpectedSchema(self, table, expected_schema):
         """Assert that a table's schema is what we expect.
@@ -107,14 +221,14 @@ class SqlModels(SqlTests):
             self.assertExpectedSchema('table_name', cols)
 
         """
-        table = self.select_table(table)
+        table = self.load_table(table)
 
         actual_schema = []
         for column in table.c:
             if isinstance(column.type, sql.Boolean):
                 default = None
-                if column._proxies[0].default:
-                    default = column._proxies[0].default.arg
+                if column.default:
+                    default = column.default.arg
                 actual_schema.append((column.name, type(column.type), default))
             elif (hasattr(column.type, 'length') and
                     not isinstance(column.type, sql.Enum)):
@@ -152,7 +266,6 @@ class SqlModels(SqlTests):
     def test_password_model(self):
         cols = (('id', sql.Integer, None),
                 ('local_user_id', sql.Integer, None),
-                ('password', sql.String, 128),
                 ('password_hash', sql.String, 255),
                 ('created_at', sql.DateTime, None),
                 ('expires_at', sql.DateTime, None),
@@ -275,24 +388,13 @@ class SqlIdentity(SqlTests,
         role_member = unit.new_role_ref()
         PROVIDERS.role_api.create_role(role_member['id'], role_member)
         PROVIDERS.assignment_api.add_role_to_user_and_project(
-            user['id'], self.tenant_bar['id'], role_member['id']
+            user['id'], self.project_bar['id'], role_member['id']
         )
         PROVIDERS.identity_api.delete_user(user['id'])
         self.assertRaises(exception.UserNotFound,
                           PROVIDERS.assignment_api.list_projects_for_user,
                           user['id'])
 
-    def test_create_null_user_name(self):
-        user = unit.new_user_ref(name=None,
-                                 domain_id=CONF.identity.default_domain_id)
-        self.assertRaises(exception.ValidationError,
-                          PROVIDERS.identity_api.create_user,
-                          user)
-        self.assertRaises(exception.UserNotFound,
-                          PROVIDERS.identity_api.get_user_by_name,
-                          user['name'],
-                          CONF.identity.default_domain_id)
-
     def test_create_user_case_sensitivity(self):
         # user name case sensitivity is down to the fact that it is marked as
         # an SQL UNIQUE column, which may not be valid for other backends, like
@@ -327,11 +429,11 @@ class SqlIdentity(SqlTests,
         role_member = unit.new_role_ref()
         PROVIDERS.role_api.create_role(role_member['id'], role_member)
         PROVIDERS.assignment_api.add_role_to_user_and_project(
-            user['id'], self.tenant_bar['id'], role_member['id']
+            user['id'], self.project_bar['id'], role_member['id']
         )
-        PROVIDERS.resource_api.delete_project(self.tenant_bar['id'])
-        tenants = PROVIDERS.assignment_api.list_projects_for_user(user['id'])
-        self.assertEqual([], tenants)
+        PROVIDERS.resource_api.delete_project(self.project_bar['id'])
+        projects = PROVIDERS.assignment_api.list_projects_for_user(user['id'])
+        self.assertEqual([], projects)
 
     def test_update_project_returns_extra(self):
         """Test for backward compatibility with an essex/folsom bug.
@@ -350,7 +452,7 @@ class SqlIdentity(SqlTests,
         project[arbitrary_key] = arbitrary_value
         ref = PROVIDERS.resource_api.create_project(project['id'], project)
         self.assertEqual(arbitrary_value, ref[arbitrary_key])
-        self.assertIsNone(ref.get('extra'))
+        self.assertNotIn('extra', ref)
 
         ref['name'] = uuid.uuid4().hex
         ref = PROVIDERS.resource_api.update_project(ref['id'], ref)
@@ -374,14 +476,14 @@ class SqlIdentity(SqlTests,
         del user["id"]
         ref = PROVIDERS.identity_api.create_user(user)
         self.assertEqual(arbitrary_value, ref[arbitrary_key])
-        self.assertIsNone(ref.get('password'))
-        self.assertIsNone(ref.get('extra'))
+        self.assertNotIn('password', ref)
+        self.assertNotIn('extra', ref)
 
         user['name'] = uuid.uuid4().hex
         user['password'] = uuid.uuid4().hex
         ref = PROVIDERS.identity_api.update_user(ref['id'], user)
-        self.assertIsNone(ref.get('password'))
-        self.assertIsNone(ref['extra'].get('password'))
+        self.assertNotIn('password', ref)
+        self.assertNotIn('password', ref['extra'])
         self.assertEqual(arbitrary_value, ref[arbitrary_key])
         self.assertEqual(arbitrary_value, ref['extra'][arbitrary_key])
 
@@ -570,6 +672,129 @@ class SqlIdentity(SqlTests,
                 negative_user['id'])
             self.assertEqual(0, len(group_refs))
 
+    def test_add_user_to_group_expiring_mapped(self):
+        self._build_fed_resource()
+        domain = self._get_domain_fixture()
+        self.config_fixture.config(group='federation',
+                                   default_authorization_ttl=5)
+        time = datetime.datetime.utcnow()
+        tick = datetime.timedelta(minutes=5)
+
+        new_group = unit.new_group_ref(domain_id=domain['id'])
+        new_group = PROVIDERS.identity_api.create_group(new_group)
+
+        fed_dict = unit.new_federated_user_ref()
+        fed_dict['idp_id'] = 'myidp'
+        fed_dict['protocol_id'] = 'mapped'
+
+        with freezegun.freeze_time(time - tick) as frozen_time:
+            user = PROVIDERS.identity_api.shadow_federated_user(
+                **fed_dict, group_ids=[new_group['id']])
+
+            PROVIDERS.identity_api.check_user_in_group(user['id'],
+                                                       new_group['id'])
+
+            # Expiration
+            frozen_time.tick(tick)
+            self.assertRaises(exception.NotFound,
+                              PROVIDERS.identity_api.check_user_in_group,
+                              user['id'],
+                              new_group['id'])
+
+            # Renewal
+            PROVIDERS.identity_api.shadow_federated_user(
+                **fed_dict, group_ids=[new_group['id']])
+            PROVIDERS.identity_api.check_user_in_group(user['id'],
+                                                       new_group['id'])
+
+    def test_add_user_to_group_expiring(self):
+        self._build_fed_resource()
+        domain = self._get_domain_fixture()
+        time = datetime.datetime.utcnow()
+        tick = datetime.timedelta(minutes=5)
+
+        new_group = unit.new_group_ref(domain_id=domain['id'])
+        new_group = PROVIDERS.identity_api.create_group(new_group)
+
+        fed_dict = unit.new_federated_user_ref()
+        fed_dict['idp_id'] = 'myidp'
+        fed_dict['protocol_id'] = 'mapped'
+        new_user = PROVIDERS.shadow_users_api.create_federated_user(
+            domain['id'], fed_dict
+        )
+
+        with freezegun.freeze_time(time - tick) as frozen_time:
+            PROVIDERS.shadow_users_api.add_user_to_group_expires(
+                new_user['id'], new_group['id'])
+
+            self.config_fixture.config(group='federation',
+                                       default_authorization_ttl=0)
+            self.assertRaises(exception.NotFound,
+                              PROVIDERS.identity_api.check_user_in_group,
+                              new_user['id'],
+                              new_group['id'])
+
+            self.config_fixture.config(group='federation',
+                                       default_authorization_ttl=5)
+            PROVIDERS.identity_api.check_user_in_group(new_user['id'],
+                                                       new_group['id'])
+
+            # Expiration
+            frozen_time.tick(tick)
+            self.assertRaises(exception.NotFound,
+                              PROVIDERS.identity_api.check_user_in_group,
+                              new_user['id'],
+                              new_group['id'])
+
+            # Renewal
+            PROVIDERS.shadow_users_api.add_user_to_group_expires(
+                new_user['id'], new_group['id'])
+            PROVIDERS.identity_api.check_user_in_group(new_user['id'],
+                                                       new_group['id'])
+
+    def test_add_user_to_group_expiring_list(self):
+        self._build_fed_resource()
+        domain = self._get_domain_fixture()
+        self.config_fixture.config(group='federation',
+                                   default_authorization_ttl=5)
+        time = datetime.datetime.utcnow()
+        tick = datetime.timedelta(minutes=5)
+
+        new_group = unit.new_group_ref(domain_id=domain['id'])
+        new_group = PROVIDERS.identity_api.create_group(new_group)
+        exp_new_group = unit.new_group_ref(domain_id=domain['id'])
+        exp_new_group = PROVIDERS.identity_api.create_group(exp_new_group)
+
+        fed_dict = unit.new_federated_user_ref()
+        fed_dict['idp_id'] = 'myidp'
+        fed_dict['protocol_id'] = 'mapped'
+        new_user = PROVIDERS.shadow_users_api.create_federated_user(
+            domain['id'], fed_dict
+        )
+
+        PROVIDERS.identity_api.add_user_to_group(new_user['id'],
+                                                 new_group['id'])
+        PROVIDERS.identity_api.check_user_in_group(new_user['id'],
+                                                   new_group['id'])
+
+        with freezegun.freeze_time(time - tick) as frozen_time:
+            PROVIDERS.shadow_users_api.add_user_to_group_expires(
+                new_user['id'], exp_new_group['id'])
+            PROVIDERS.identity_api.check_user_in_group(new_user['id'],
+                                                       new_group['id'])
+
+            groups = PROVIDERS.identity_api.list_groups_for_user(
+                new_user['id'])
+            self.assertEqual(len(groups), 2)
+            for group in groups:
+                if group.get('membership_expires_at'):
+                    self.assertEqual(group['membership_expires_at'], time)
+
+            frozen_time.tick(tick)
+            groups = PROVIDERS.identity_api.list_groups_for_user(
+                new_user['id'])
+            self.assertEqual(len(groups), 1)
+
     def test_storing_null_domain_id_in_project_ref(self):
         """Test the special storage of domain_id=None in sql resource driver.
 
@@ -684,8 +909,10 @@ class SqlIdentity(SqlTests,
                               ref_id)
 
             # Deleting list of projects that includes a non-existing project
-            # should be silent
-            driver.delete_projects_from_ids([ref_id])
+            # should be silent. The root domain <<keystone.domain.root>> can't
+            # be deleted.
+            if ref_id != resource.NULL_DOMAIN_ID:
+                driver.delete_projects_from_ids([ref_id])
 
         _exercise_project_api(uuid.uuid4().hex)
         _exercise_project_api(resource.NULL_DOMAIN_ID)
@@ -726,17 +953,23 @@ class SqlIdentity(SqlTests,
         # users fetched.
         self.assertNotEqual(len(first_call_users), len(second_call_users))
         self.assertEqual(first_call_counter, counter.calls)
+        self.assertEqual(3, counter.calls)
 
     def test_check_project_depth(self):
-        # create a 3 level project tree
-        ref = unit.new_project_ref(domain_id=CONF.identity.default_domain_id)
-        PROVIDERS.resource_api.create_project(ref['id'], ref)
-        ref_1 = unit.new_project_ref(domain_id=CONF.identity.default_domain_id,
-                                     parent_id=ref['id'])
-        PROVIDERS.resource_api.create_project(ref_1['id'], ref_1)
-        ref_2 = unit.new_project_ref(domain_id=CONF.identity.default_domain_id,
-                                     parent_id=ref_1['id'])
-        PROVIDERS.resource_api.create_project(ref_2['id'], ref_2)
+        # Create a 3 level project tree:
+        #
+        # default_domain
+        #       |
+        #   project_1
+        #       |
+        #   project_2
+        project_1 = unit.new_project_ref(
+            domain_id=CONF.identity.default_domain_id)
+        PROVIDERS.resource_api.create_project(project_1['id'], project_1)
+        project_2 = unit.new_project_ref(
+            domain_id=CONF.identity.default_domain_id,
+            parent_id=project_1['id'])
+        PROVIDERS.resource_api.create_project(project_2['id'], project_2)
 
         # if max_depth is None or >= current project depth, return nothing.
         resp = PROVIDERS.resource_api.check_project_depth(max_depth=None)
@@ -768,21 +1001,21 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
     _legacy_endpoint_id_in_endpoint = True
     _enabled_default_to_true_when_creating_endpoint = True
 
-    def test_catalog_ignored_malformed_urls(self):
+    def test_get_v3_catalog_project_non_exist(self):
         service = unit.new_service_ref()
         PROVIDERS.catalog_api.create_service(service['id'], service)
 
-        malformed_url = "http://192.168.1.104:8774/v2/$(tenant)s"
+        malformed_url = "http://192.168.1.104:8774/v2/$(project)s"
         endpoint = unit.new_endpoint_ref(service_id=service['id'],
                                          url=malformed_url,
                                          region_id=None)
         PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint.copy())
+        self.assertRaises(exception.ProjectNotFound,
+                          PROVIDERS.catalog_api.get_v3_catalog,
+                          'fake-user',
+                          'fake-project')
 
-        # NOTE(dstanek): there are no valid URLs, so nothing is in the catalog
-        catalog = PROVIDERS.catalog_api.get_catalog('fake-user', 'fake-tenant')
-        self.assertEqual({}, catalog)
-
-    def test_get_catalog_with_empty_public_url(self):
+    def test_get_v3_catalog_with_empty_public_url(self):
         service = unit.new_service_ref()
         PROVIDERS.catalog_api.create_service(service['id'], service)
 
@@ -790,13 +1023,12 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
                                          region_id=None)
         PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint.copy())
 
-        catalog = PROVIDERS.catalog_api.get_catalog('user', 'tenant')
-        catalog_endpoint = catalog[endpoint['region_id']][service['type']]
+        catalog = PROVIDERS.catalog_api.get_v3_catalog(self.user_foo['id'],
+                                                       self.project_bar['id'])
+        catalog_endpoint = catalog[0]
         self.assertEqual(service['name'], catalog_endpoint['name'])
-        self.assertEqual(endpoint['id'], catalog_endpoint['id'])
-        self.assertEqual('', catalog_endpoint['publicURL'])
-        self.assertIsNone(catalog_endpoint.get('adminURL'))
-        self.assertIsNone(catalog_endpoint.get('internalURL'))
+        self.assertEqual(service['id'], catalog_endpoint['id'])
+        self.assertEqual([], catalog_endpoint['endpoints'])
 
     def test_create_endpoint_region_returns_not_found(self):
         service = unit.new_service_ref()
@@ -857,7 +1089,7 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
                           region['id'])
 
     def test_v3_catalog_domain_scoped_token(self):
-        # test the case that tenant_id is None.
+        # test the case that project_id is None.
         srv_1 = unit.new_service_ref()
         PROVIDERS.catalog_api.create_service(srv_1['id'], srv_1)
         endpoint_1 = unit.new_endpoint_ref(service_id=srv_1['id'],
@@ -895,10 +1127,10 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
         # create endpoint-project association.
         PROVIDERS.catalog_api.add_endpoint_to_project(
             endpoint_1['id'],
-            self.tenant_bar['id'])
+            self.project_bar['id'])
 
         catalog_ref = PROVIDERS.catalog_api.get_v3_catalog(
-            uuid.uuid4().hex, self.tenant_bar['id']
+            uuid.uuid4().hex, self.project_bar['id']
         )
         self.assertThat(catalog_ref, matchers.HasLength(1))
         self.assertThat(catalog_ref[0]['endpoints'], matchers.HasLength(1))
@@ -920,7 +1152,7 @@ class SqlCatalog(SqlTests, catalog_tests.CatalogTests):
         PROVIDERS.catalog_api.create_service(srv_2['id'], srv_2)
 
         catalog_ref = PROVIDERS.catalog_api.get_v3_catalog(
-            uuid.uuid4().hex, self.tenant_bar['id']
+            uuid.uuid4().hex, self.project_bar['id']
         )
         self.assertThat(catalog_ref, matchers.HasLength(2))
         srv_id_list = [catalog_ref[0]['id'], catalog_ref[1]['id']]
