@@ -19,8 +19,6 @@ from cryptography import fernet
 import msgpack
 from oslo_log import log
 from oslo_utils import timeutils
-import six
-from six.moves import map
 
 from keystone.auth import plugins as auth_plugins
 from keystone.common import fernet_utils as utils
@@ -71,8 +69,8 @@ class TokenFormatter(object):
     def pack(self, payload):
         """Pack a payload for transport as a token.
 
-        :type payload: six.binary_type
-        :rtype: six.text_type
+        :type payload: bytes
+        :rtype: str
 
         """
         # base64 padding (if any) is not URL-safe
@@ -81,8 +79,8 @@ class TokenFormatter(object):
     def unpack(self, token):
         """Unpack a token, and validate the payload.
 
-        :type token: six.text_type
-        :rtype: six.binary_type
+        :type token: str
+        :rtype: bytes
 
         """
         token = TokenFormatter.restore_padding(token)
@@ -91,14 +89,14 @@ class TokenFormatter(object):
             return self.crypto.decrypt(token.encode('utf-8'))
         except fernet.InvalidToken:
             raise exception.ValidationError(
-                _('This is not a recognized Fernet token %s') % token)
+                _('Could not recognize Fernet token'))
 
     @classmethod
     def restore_padding(cls, token):
         """Restore padding based on token size.
 
         :param token: token to restore padding on
-        :type token: six.text_type
+        :type token: str
         :returns: token with correct padding
 
         """
@@ -113,14 +111,14 @@ class TokenFormatter(object):
     def creation_time(cls, fernet_token):
         """Return the creation time of a valid Fernet token.
 
-        :type fernet_token: six.text_type
+        :type fernet_token: str
 
         """
         fernet_token = TokenFormatter.restore_padding(fernet_token)
-        # fernet_token is six.text_type
+        # fernet_token is str
 
         # Fernet tokens are base64 encoded, so we need to unpack them first
-        # urlsafe_b64decode() requires six.binary_type
+        # urlsafe_b64decode() requires bytes
         token_bytes = base64.urlsafe_b64decode(fernet_token.encode('utf-8'))
 
         # slice into the byte array to get just the timestamp
@@ -135,23 +133,12 @@ class TokenFormatter(object):
 
         return issued_at
 
-    def create_token(self, user_id, expires_at, audit_ids, methods=None,
-                     system=None, domain_id=None, project_id=None,
-                     trust_id=None, federated_group_ids=None,
+    def create_token(self, user_id, expires_at, audit_ids, payload_class,
+                     methods=None, system=None, domain_id=None,
+                     project_id=None, trust_id=None, federated_group_ids=None,
                      identity_provider_id=None, protocol_id=None,
                      access_token_id=None, app_cred_id=None):
         """Given a set of payload attributes, generate a Fernet token."""
-        for payload_class in PAYLOAD_CLASSES:
-            if payload_class.create_arguments_apply(
-                    project_id=project_id, domain_id=domain_id,
-                    system=system, trust_id=trust_id,
-                    federated_group_ids=federated_group_ids,
-                    identity_provider_id=identity_provider_id,
-                    protocol_id=protocol_id,
-                    access_token_id=access_token_id,
-                    app_cred_id=app_cred_id):
-                break
-
         version = payload_class.version
         payload = payload_class.assemble(
             user_id, methods, system, project_id, domain_id, expires_at,
@@ -179,14 +166,22 @@ class TokenFormatter(object):
     def validate_token(self, token):
         """Validate a Fernet token and returns the payload attributes.
 
-        :type token: six.text_type
+        :type token: str
 
         """
         serialized_payload = self.unpack(token)
-        versioned_payload = msgpack.unpackb(serialized_payload)
+        # TODO(melwitt): msgpack changed their data format in version 1.0, so
+        # in order to support a rolling upgrade, we must pass raw=True to
+        # support the old format. The try-except may be removed once the
+        # N-1 release no longer supports msgpack < 1.0.
+        try:
+            versioned_payload = msgpack.unpackb(serialized_payload)
+        except UnicodeDecodeError:
+            versioned_payload = msgpack.unpackb(serialized_payload, raw=True)
+
         version, payload = versioned_payload[0], versioned_payload[1:]
 
-        for payload_class in PAYLOAD_CLASSES:
+        for payload_class in _PAYLOAD_CLASSES:
             if version == payload_class.version:
                 (user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -223,17 +218,6 @@ class BasePayload(object):
     # each payload variant should have a unique version
     version = None
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        """Check the arguments to see if they apply to this payload variant.
-
-        :returns: True if the arguments indicate that this payload class is
-                  needed for the token otherwise returns False.
-        :rtype: bool
-
-        """
-        raise NotImplementedError()
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -335,29 +319,31 @@ class BasePayload(object):
         """
         try:
             return (True, cls.convert_uuid_hex_to_bytes(value))
-        except ValueError:
-            # this might not be a UUID, depending on the situation (i.e.
-            # federation)
+        except (ValueError, TypeError):
+            # ValueError: this might not be a UUID, depending on the
+            # situation (i.e. federation)
+            # TypeError: the provided value may be binary encoded
+            # in which case just return the value (i.e. Python 3)
             return (False, value)
 
     @classmethod
     def base64_encode(cls, s):
         """Encode a URL-safe string.
 
-        :type s: six.text_type
-        :rtype: six.text_type
+        :type s: str
+        :rtype: str
 
         """
-        # urlsafe_b64encode() returns six.binary_type so need to convert to
-        # six.text_type, might as well do it before stripping.
+        # urlsafe_b64encode() returns bytes so need to convert to
+        # str, might as well do it before stripping.
         return base64.urlsafe_b64encode(s).decode('utf-8').rstrip('=')
 
     @classmethod
     def random_urlsafe_str_to_bytes(cls, s):
-        """Convert a string from :func:`random_urlsafe_str()` to six.binary_type.
+        """Convert string from :func:`random_urlsafe_str()` to bytes.
 
-        :type s: six.text_type
-        :rtype: six.binary_type
+        :type s: str
+        :rtype: bytes
 
         """
         # urlsafe_b64decode() requires str, unicode isn't accepted.
@@ -366,14 +352,26 @@ class BasePayload(object):
         # restore the padding (==) at the end of the string
         return base64.urlsafe_b64decode(s + '==')
 
+    @classmethod
+    def _convert_or_decode(cls, is_stored_as_bytes, value):
+        """Convert a value to text type, translating uuid -> hex if required.
+
+        :param is_stored_as_bytes: whether value is already bytes
+        :type is_stored_as_bytes: boolean
+        :param value: value to attempt to convert to bytes
+        :type value: str or bytes
+        :rtype: str
+        """
+        if is_stored_as_bytes:
+            return cls.convert_uuid_bytes_to_hex(value)
+        elif isinstance(value, bytes):
+            return value.decode('utf-8')
+        return value
+
 
 class UnscopedPayload(BasePayload):
     version = 0
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return True
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -389,8 +387,7 @@ class UnscopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         expires_at_str = cls._convert_float_to_time_string(payload[2])
         audit_ids = list(map(cls.base64_encode, payload[3]))
@@ -412,10 +409,6 @@ class UnscopedPayload(BasePayload):
 class DomainScopedPayload(BasePayload):
     version = 1
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['domain_id']
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -439,14 +432,13 @@ class DomainScopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         try:
             domain_id = cls.convert_uuid_bytes_to_hex(payload[2])
         except ValueError:
             # the default domain ID is configurable, and probably isn't a UUID
-            if six.PY3 and isinstance(payload[2], six.binary_type):
+            if isinstance(payload[2], bytes):
                 payload[2] = payload[2].decode('utf-8')
             if payload[2] == CONF.identity.default_domain_id:
                 domain_id = payload[2]
@@ -471,10 +463,6 @@ class DomainScopedPayload(BasePayload):
 class ProjectScopedPayload(BasePayload):
     version = 2
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['project_id']
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -491,12 +479,10 @@ class ProjectScopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         (is_stored_as_bytes, project_id) = payload[2]
-        if is_stored_as_bytes:
-            project_id = cls.convert_uuid_bytes_to_hex(project_id)
+        project_id = cls._convert_or_decode(is_stored_as_bytes, project_id)
         expires_at_str = cls._convert_float_to_time_string(payload[3])
         audit_ids = list(map(cls.base64_encode, payload[4]))
         system = None
@@ -516,10 +502,6 @@ class ProjectScopedPayload(BasePayload):
 class TrustScopedPayload(BasePayload):
     version = 3
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['trust_id']
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -539,12 +521,10 @@ class TrustScopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         (is_stored_as_bytes, project_id) = payload[2]
-        if is_stored_as_bytes:
-            project_id = cls.convert_uuid_bytes_to_hex(project_id)
+        project_id = cls._convert_or_decode(is_stored_as_bytes, project_id)
         expires_at_str = cls._convert_float_to_time_string(payload[3])
         audit_ids = list(map(cls.base64_encode, payload[4]))
         trust_id = cls.convert_uuid_bytes_to_hex(payload[5])
@@ -564,10 +544,6 @@ class TrustScopedPayload(BasePayload):
 class FederatedUnscopedPayload(BasePayload):
     version = 4
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['federated_group_ids']
-
     @classmethod
     def pack_group_id(cls, group_dict):
         return cls.attempt_convert_uuid_hex_to_bytes(group_dict['id'])
@@ -575,8 +551,7 @@ class FederatedUnscopedPayload(BasePayload):
     @classmethod
     def unpack_group_id(cls, group_id_in_bytes):
         (is_stored_as_bytes, group_id) = group_id_in_bytes
-        if is_stored_as_bytes:
-            group_id = cls.convert_uuid_bytes_to_hex(group_id)
+        group_id = cls._convert_or_decode(is_stored_as_bytes, group_id)
         return {'id': group_id}
 
     @classmethod
@@ -598,17 +573,13 @@ class FederatedUnscopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         group_ids = list(map(cls.unpack_group_id, payload[2]))
         (is_stored_as_bytes, idp_id) = payload[3]
-        if is_stored_as_bytes:
-            idp_id = cls.convert_uuid_bytes_to_hex(idp_id)
-        else:
-            idp_id = idp_id.decode('utf-8')
+        idp_id = cls._convert_or_decode(is_stored_as_bytes, idp_id)
         protocol_id = payload[4]
-        if isinstance(protocol_id, six.binary_type):
+        if isinstance(protocol_id, bytes):
             protocol_id = protocol_id.decode('utf-8')
         expires_at_str = cls._convert_float_to_time_string(payload[5])
         audit_ids = list(map(cls.base64_encode, payload[6]))
@@ -647,12 +618,10 @@ class FederatedScopedPayload(FederatedUnscopedPayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         (is_stored_as_bytes, scope_id) = payload[2]
-        if is_stored_as_bytes:
-            scope_id = cls.convert_uuid_bytes_to_hex(scope_id)
+        scope_id = cls._convert_or_decode(is_stored_as_bytes, scope_id)
         project_id = (
             scope_id
             if cls.version == FederatedProjectScopedPayload.version else None)
@@ -661,9 +630,10 @@ class FederatedScopedPayload(FederatedUnscopedPayload):
             if cls.version == FederatedDomainScopedPayload.version else None)
         group_ids = list(map(cls.unpack_group_id, payload[3]))
         (is_stored_as_bytes, idp_id) = payload[4]
-        if is_stored_as_bytes:
-            idp_id = cls.convert_uuid_bytes_to_hex(idp_id)
+        idp_id = cls._convert_or_decode(is_stored_as_bytes, idp_id)
         protocol_id = payload[5]
+        if isinstance(protocol_id, bytes):
+            protocol_id = protocol_id.decode('utf-8')
         expires_at_str = cls._convert_float_to_time_string(payload[6])
         audit_ids = list(map(cls.base64_encode, payload[7]))
         system = None
@@ -678,26 +648,14 @@ class FederatedScopedPayload(FederatedUnscopedPayload):
 class FederatedProjectScopedPayload(FederatedScopedPayload):
     version = 5
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['project_id'] and kwargs['federated_group_ids']
-
 
 class FederatedDomainScopedPayload(FederatedScopedPayload):
     version = 6
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['domain_id'] and kwargs['federated_group_ids']
-
 
 class OauthScopedPayload(BasePayload):
     version = 7
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['access_token_id']
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -717,15 +675,13 @@ class OauthScopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         (is_stored_as_bytes, project_id) = payload[2]
-        if is_stored_as_bytes:
-            project_id = cls.convert_uuid_bytes_to_hex(project_id)
+        project_id = cls._convert_or_decode(is_stored_as_bytes, project_id)
         (is_stored_as_bytes, access_token_id) = payload[3]
-        if is_stored_as_bytes:
-            access_token_id = cls.convert_uuid_bytes_to_hex(access_token_id)
+        access_token_id = cls._convert_or_decode(is_stored_as_bytes,
+                                                 access_token_id)
         expires_at_str = cls._convert_float_to_time_string(payload[4])
         audit_ids = list(map(cls.base64_encode, payload[5]))
         system = None
@@ -745,10 +701,6 @@ class OauthScopedPayload(BasePayload):
 class SystemScopedPayload(BasePayload):
     version = 8
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['system']
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -764,8 +716,7 @@ class SystemScopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         system = payload[2]
         expires_at_str = cls._convert_float_to_time_string(payload[3])
@@ -787,10 +738,6 @@ class SystemScopedPayload(BasePayload):
 class ApplicationCredentialScopedPayload(BasePayload):
     version = 9
 
-    @classmethod
-    def create_arguments_apply(cls, **kwargs):
-        return kwargs['app_cred_id']
-
     @classmethod
     def assemble(cls, user_id, methods, system, project_id, domain_id,
                  expires_at, audit_ids, trust_id, federated_group_ids,
@@ -809,12 +756,10 @@ class ApplicationCredentialScopedPayload(BasePayload):
     @classmethod
     def disassemble(cls, payload):
         (is_stored_as_bytes, user_id) = payload[0]
-        if is_stored_as_bytes:
-            user_id = cls.convert_uuid_bytes_to_hex(user_id)
+        user_id = cls._convert_or_decode(is_stored_as_bytes, user_id)
         methods = auth_plugins.convert_integer_to_method_list(payload[1])
         (is_stored_as_bytes, project_id) = payload[2]
-        if is_stored_as_bytes:
-            project_id = cls.convert_uuid_bytes_to_hex(project_id)
+        project_id = cls._convert_or_decode(is_stored_as_bytes, project_id)
         expires_at_str = cls._convert_float_to_time_string(payload[3])
         audit_ids = list(map(cls.base64_encode, payload[4]))
         system = None
@@ -825,30 +770,22 @@ class ApplicationCredentialScopedPayload(BasePayload):
         protocol_id = None
         access_token_id = None
         (is_stored_as_bytes, app_cred_id) = payload[5]
-        if is_stored_as_bytes:
-            app_cred_id = cls.convert_uuid_bytes_to_hex(app_cred_id)
+        app_cred_id = cls._convert_or_decode(is_stored_as_bytes, app_cred_id)
         return (user_id, methods, system, project_id, domain_id,
                 expires_at_str, audit_ids, trust_id, federated_group_ids,
                 identity_provider_id, protocol_id, access_token_id,
                 app_cred_id)
 
 
-# For now, the order of the classes in the following list is important. This
-# is because the way they test that the payload applies to them in
-# the create_arguments_apply method requires that the previous ones rejected
-# the payload arguments. For example, UnscopedPayload must be last since it's
-# the catch-all after all the other payloads have been checked.
-# TODO(blk-u): Clean up the create_arguments_apply methods so that they don't
-# depend on the previous classes then these can be in any order.
-PAYLOAD_CLASSES = [
-    OauthScopedPayload,
+_PAYLOAD_CLASSES = [
+    UnscopedPayload,
+    DomainScopedPayload,
+    ProjectScopedPayload,
     TrustScopedPayload,
+    FederatedUnscopedPayload,
     FederatedProjectScopedPayload,
     FederatedDomainScopedPayload,
-    FederatedUnscopedPayload,
-    ApplicationCredentialScopedPayload,
-    ProjectScopedPayload,
-    DomainScopedPayload,
+    OauthScopedPayload,
     SystemScopedPayload,
-    UnscopedPayload,
+    ApplicationCredentialScopedPayload,
 ]
